@@ -1,18 +1,9 @@
 # Copyright (c) 2007-2008 Roy Marples <roy@marples.name>
 # Released under the 2-clause BSD license.
 
-_ip()
-{
-	if [ -x /bin/ip ]; then
-		echo /bin/ip
-	else
-		echo /sbin/ip
-	fi
-}
-
 iproute2_depend()
 {
-	program $(_ip)
+	program ip
 	provide interface
 	after ifconfig
 }
@@ -29,7 +20,7 @@ _down()
 
 _exists()
 {
-	grep -Eq "^[[:space:]]*${IFACE}:" /proc/net/dev
+	[ -e /sys/class/net/"$IFACE" ]
 }
 
 _ifindex()
@@ -110,31 +101,54 @@ _add_address()
 		ip addr add "$@" dev "${IFACE}" 2>/dev/null
 		return 0
 	fi
-
+	local x
 	local address netmask broadcast peer anycast label scope
 	local valid_lft preferred_lft home nodad
+	local confflaglist
 	address="$1" ; shift
 	while [ -n "$*" ]; do
-		case "$1" in
-			netmask)
-				netmask="/$(_netmask2cidr "$2")" ; shift ; shift ;;
-			broadcast|brd)
-				broadcast="broadcast $2" ; shift ; shift ;;
-			pointopoint|pointtopoint|peer)
-				peer="peer $2" ; shift ; shift ;;
-			anycast|label|scope|valid_lft|preferred_lft)
-				eval "$1=$2" ; shift ; shift ;;
-			home|nodad)
-				eval "$1=$1" ; shift ;;
+		x=$1 ; shift
+		case "$x" in
+			netmask|ne*)
+				netmask="/$(_netmask2cidr "$1")" ; shift ;;
+			broadcast|brd|br*)
+				broadcast="$1" ; shift ;;
+			pointopoint|pointtopoint|peer|po*|pe*)
+				peer="$1" ; shift ;;
+			anycast|label|scope|valid_lft|preferred_lft|a*|l*|s*|v*|pr*)
+				case $x in
+					a*) x=anycast ;;
+					l*) x=label ;;
+					s*) x=scope ;;
+					v*) x=valid_lft ;;
+					pr*) x=preferred_lft ;;
+				esac
+				eval "$x=$1" ; shift ;;
+			home|nodad|h*|no*)
+				case $x in h*) x=home ;; n*) x=nodad ;; esac
+				# FIXME: If we need to reorder these, this will take more code
+				confflaglist="${confflaglist} $x" ; ;;
+			*)
+				ewarn "Unknown argument to config_$IFACE: $x"
 		esac
 	done
 
 	# Always scope lo addresses as host unless specified otherwise
 	if [ "${IFACE}" = "lo" ]; then
-		[ -z "$scope" ] && scope="scope host"
+		[ -z "$scope" ] && scope="host"
 	fi
 
-	set -- "${address}${netmask}" $peer $broadcast $anycast $label $scope dev "${IFACE}" $valid_lft $preferred_lft $home $nodad
+	# figure out the broadcast address if it is not specified
+	# This must NOT be set for IPv6 addresses
+	if [ "${address#*:}" = "${address}" ]; then
+		[ -z "$broadcast" ] && broadcast="+"
+	elif [ -n "$broadcast" ]; then
+		eerror "Broadcast keywords are not valid with IPv6 addresses"
+		return 1
+	fi
+
+	# This must appear on a single line, continuations cannot be used
+	set -- "${address}${netmask}" ${peer:+peer} ${peer} ${broadcast:+broadcast} ${broadcast} ${anycast:+anycast} ${anycast} ${label:+label} ${label} ${scope:+scope} ${scope} dev "${IFACE}" ${valid_lft:+valid_lft} $valid_lft ${preferred_lft:+preferred_lft} $preferred_lft $confflaglist
 	veinfo ip addr add "$@"
 	ip addr add "$@"
 }
@@ -146,6 +160,12 @@ _add_route()
 	if [ "$1" = "-A" -o "$1" = "-f" -o "$1" = "-family" ]; then
 		family="-f $2"
 		shift; shift
+	elif [ "$1" = "-4" ]; then
+	    family="-f inet"
+		shift
+	elif [ "$1" = "-6" ]; then
+	    family="-f inet6"
+		shift
 	fi
 
 	if [ $# -eq 3 ]; then
@@ -208,7 +228,13 @@ _trim() {
 # This is our interface to Routing Policy Database RPDB
 # This allows for advanced routing tricks
 _ip_rule_runner() {
-	local cmd rules OIFS="${IFS}"
+	local cmd rules OIFS="${IFS}" family
+	if [ "$1" = "-4" -o "$1" = "-6" ]; then
+		family="$1"
+		shift
+	else
+		family="-4"
+	fi
 	cmd="$1"
 	rules="$2"
 	veindent
@@ -218,7 +244,7 @@ _ip_rule_runner() {
 		ruN="$(_trim "${ru}")"
 		[ -z "${ruN}" ] && continue
 		vebegin "${cmd} ${ruN}"
-		ip rule ${cmd} ${ru}
+		ip $family rule ${cmd} ${ru}
 		veend $?
 		local IFS="$__IFS"
 	done
@@ -274,15 +300,30 @@ iproute2_post_start()
 	if [ -e /proc/net/route ]; then
 		local rules="$(_get_array "rules_${IFVAR}")"
 		if [ -n "${rules}" ]; then
-			if ! ip rule list | grep -q "^"; then
+			if ! ip -4 rule list | grep -q "^"; then
 				eerror "IP Policy Routing (CONFIG_IP_MULTIPLE_TABLES) needed for ip rule"
 			else
 				service_set_value "ip_rule" "${rules}"
-				einfo "Adding RPDB rules"
-				_ip_rule_runner add "${rules}"
+				einfo "Adding IPv4 RPDB rules"
+				_ip_rule_runner -4 add "${rules}"
 			fi
 		fi
-		ip route flush table cache dev "${IFACE}"
+		ip -4 route flush table cache dev "${IFACE}"
+	fi
+
+	# Kernel may not have IPv6 built in
+	if [ -e /proc/net/ipv6_route ]; then
+		local rules="$(_get_array "rules6_${IFVAR}")"
+		if [ -n "${rules}" ]; then
+			if ! ip -6 rule list | grep -q "^"; then
+				eerror "IPv6 Policy Routing (CONFIG_IPV6_MULTIPLE_TABLES) needed for ip rule"
+			else
+				service_set_value "ip6_rule" "${rules}"
+				einfo "Adding IPv6 RPDB rules"
+				_ip_rule_runner -6 add "${rules}"
+			fi
+		fi
+		ip -6 route flush table cache dev "${IFACE}"
 	fi
 
 	if _iproute2_ipv6_tentative; then
@@ -305,13 +346,27 @@ iproute2_post_stop()
 	if [ -e /proc/net/route ]; then
 		local rules="$(service_get_value "ip_rule")"
 		if [ -n "${rules}" ]; then
-			einfo "Removing RPDB rules"
-			_ip_rule_runner del "${rules}"
+			einfo "Removing IPv4 RPDB rules"
+			_ip_rule_runner -4 del "${rules}"
 		fi
 
 		# Only do something if the interface actually exist
 		if _exists; then
-			ip route flush table cache dev "${IFACE}"
+			ip -4 route flush table cache dev "${IFACE}"
+		fi
+	fi
+
+	# Kernel may not have IPv6 built in
+	if [ -e /proc/net/ipv6_route ]; then
+		local rules="$(service_get_value "ip6_rule")"
+		if [ -n "${rules}" ]; then
+			einfo "Removing IPv6 RPDB rules"
+			_ip_rule_runner -6 del "${rules}"
+		fi
+
+		# Only do something if the interface actually exist
+		if _exists; then
+			ip -6 route flush table cache dev "${IFACE}"
 		fi
 	fi
 
@@ -324,3 +379,26 @@ iproute2_post_stop()
 		fi
 	fi
 }
+
+# Is the interface administratively/operationally up?
+# The 'UP' status in ifconfig/iproute2 is the administrative status
+# Operational state is available in iproute2 output as 'state UP', or the
+# operstate sysfs variable.
+# 0: up
+# 1: down
+# 2: invalid arguments
+is_admin_up()
+{
+	local iface="$1"
+	[ -z "$iface" ] && iface="$IFACE"
+	ip link show dev $iface | \
+	sed -n '1,1{ /[<,]UP[,>]/{ q 0 }}; q 1; '
+}
+
+is_oper_up()
+{
+	local iface="$1"
+	[ -z "$iface" ] && iface="$IFACE"
+	read state </sys/class/net/"${iface}"/operstate
+	[ "x$state" = "up" ]
+}
